#include "config.h"
#include <OctoPrintAPI.h>
#include <WiFiClient.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "tunes.h"

void playTune(int tuneNumber);
WiFiClient client;

OctoprintApi octoprint(client, (const char*)OCTOPRINT_HOST, OCTOPRINT_PORT, OCTOPRINT_API_KEY);
unsigned long octoprint_mtbs = OCTOPRINT_API_REFRESH_INTERVAL;
unsigned long octoprint_lasttime = 0;

bool displayOctoPrintVersion(Adafruit_SSD1306& display) {
    const int maxRetries = 3;  // Maximum number of attempts
    int attempts = 0;

    // Initial display of connecting message with host and port
    display.clearDisplay();
    display.setCursor(0, 0);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.print("Connecting to");
    display.setCursor(0, 10);
    display.print("OctoPrint");
    display.setCursor(0, 25);
    display.print("Host: ");
    display.print(OCTOPRINT_HOST);
    display.setCursor(0, 35);
    display.print("Port: ");
    display.print(OCTOPRINT_PORT);
    display.display();

    delay(500);  // Delay to allow message display before attempting connection

    // Attempt to connect up to maxRetries times
    while (attempts < maxRetries) {
        if (octoprint.getOctoprintVersion()) {
            // Connection successful; display version information
            display.clearDisplay();
            display.setCursor(0, 0);
            display.setTextSize(1);
            display.setTextColor(WHITE);
            display.print("OctoPrint Version:");
            display.setCursor(0, 10);
            display.setTextSize(2);
            display.print(octoprint.octoprintVer.octoprintServer);
            display.display();
            playTune(2);  // Success tune
            return true;  // Exit function, indicating success
        }

        attempts++;  // Increment attempt count

        // If the connection attempt fails, clear the display and show retry status
        display.clearDisplay();
        display.setCursor(0, 0);
        display.setTextSize(1);
        display.setTextColor(WHITE);
        display.print("Retry ");
        display.print(attempts);
        display.print(" of ");
        display.print(maxRetries);
        display.setCursor(0, 10);
        display.print("Failed to connect.");
        display.display();
        delay(5000);  // Wait before retrying
    }

    // If all attempts fail, display final failure message
    display.clearDisplay();
    display.setCursor(0, 0);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.print("Failed to get");
    display.setCursor(0, 10);
    display.print("OctoPrint version.");
    display.display();
    playTune(3);  // Failure tune
    return false;
}

int scrollPos = 0;         // Global variable to keep track of the scroll position
int scrollDirection = 1;       // 1 for left, -1 for right
unsigned long lastScrollTime = 0; // To control the scroll speed
const int scrollDelay = 100; // Delay in milliseconds between scroll updates

bool fetchPrintingStatus(Adafruit_SSD1306& display) {
    // Fetch printer statistics and update the display every octoprint_mtbs milliseconds
    if ((millis() - octoprint_lasttime > octoprint_mtbs || octoprint_lasttime == 0) && !is_display_off) {
        if (octoprint.getPrinterStatistics() && octoprint.getPrintJob()) {
            // Update last fetch time
            octoprint_lasttime = millis();
        } else {
            Serial.println("Failed to fetch printer statistics.");
            return false;
        }
    }

    // Regardless of whether the fetch was successful, continue with the display updates
    display.clearDisplay();
    display.setCursor(0, 0);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    
    // Display lines
    display.println(octoprint.printerStats.printerState);
    display.drawLine(0, 10, 127, 10, WHITE);
    display.drawLine(63, 10, 63, 22, WHITE);
    display.drawLine(0, 22, 127, 22, WHITE);

    // Display nozzle and bed temperature
    display.drawBitmap(10, 13, bitmap_nozzle, 8, 8, WHITE);
    display.setCursor(21, 13);
    display.print(octoprint.printerStats.printerTool0TempActual);
    display.drawBitmap(74, 13, bitmap_bed, 8, 8, WHITE);
    display.setCursor(85, 13);
    display.print(octoprint.printerStats.printerBedTempActual);

    // Estimated print time in HH:MM:SS format
    int estrunHours = octoprint.printJob.estimatedPrintTime / 3600;
    int estsecsRemaining = octoprint.printJob.estimatedPrintTime % 3600;
    int estrunMinutes = estsecsRemaining / 60;
    int estrunSeconds = estsecsRemaining % 60;
    char estbuf[31];
    sprintf(estbuf, "%02d:%02d:%02d", estrunHours, estrunMinutes, estrunSeconds);
    display.setCursor(80, 0);
    display.println(estbuf);

    // Display and scroll the file name
    const char* fileName = octoprint.printJob.jobFileName.c_str(); // Convert to const char*
    int fileNameWidth = strlen(fileName) * 6;                // Approximate width of the file name in pixels
    int displayWidth = 127;

    // Clear the area for the file name to avoid overlap
    display.fillRect(0, 26, displayWidth, 11, BLACK);

    // Check if it’s time to scroll based on the delay
    if(fileNameWidth > displayWidth) {
        if (millis() - lastScrollTime > scrollDelay) {
            // Update scroll position based on the current direction
            scrollPos += scrollDirection;
            lastScrollTime = millis();

            // Reverse direction if the text reaches either edge
            if (scrollPos > 0) { // Reached the left edge
                scrollDirection = -1; // Start moving left
            } else if (scrollPos < -fileNameWidth + displayWidth) { // Reached the right edge
                scrollDirection = 1; // Start moving right
            }
        }
    }
    // Draw the file name with current scroll position
    display.setCursor(scrollPos, 27);
    display.print(fileName);

    // Clear the area for "Time left" display before printing
    display.fillRect(0, 35, displayWidth, 9, BLACK); // Clear area for "Time left"
    // Print time left in HH:MM:SS
    int runHours = octoprint.printJob.progressPrintTimeLeft / 3600;
    int secsRemaining = octoprint.printJob.progressPrintTimeLeft % 3600;
    int runMinutes = secsRemaining / 60;
    int runSeconds = secsRemaining % 60;
    char buf[31];
    sprintf(buf, "Time left: %02d:%02d:%02d", runHours, runMinutes, runSeconds);
    display.setCursor(0, 42);
    display.println(buf);

    // Progress bar and percentage
    const float temp_percent = floor(octoprint.printJob.progressCompletion * 100) / 100;
    int barY = 52;      // Position just above the bottom of a 64-pixel-high display
    int barWidth = 127; // Full width of the display
    int barHeight = 12; // Set the height of the progress bar to 12 pixels
    int filledWidth = (temp_percent / 100.0) * barWidth;

    // Draw the progress bar
    display.drawRect(0, barY, barWidth, barHeight, WHITE);    // Outline of the bar
    display.fillRect(0, barY, filledWidth, barHeight, WHITE); // Filled portion
    
    // Display centered percentage text in the progress bar
    char percentText[6];
    sprintf(percentText, "%.0f%%", temp_percent);
    int textWidth = strlen(percentText) * 6;  // Approximate text width
    int textX = (barWidth - textWidth) / 2;   // Center the text horizontally
    int textY = barY + (barHeight - 8) / 2;   // Center the text vertically within the bar
    
    // Clear the area for text and print centered percentage
    display.fillRect(textX - 1, textY -1, textWidth + 2, 10, BLACK);
    display.setCursor(textX, textY);
    display.print(percentText);
    display.drawLine(0, 38, 127, 38, WHITE);

    display.display();

    return true; // Return true to indicate the display was updated
}
